#!/usr/bin/pypy
# -*- coding: utf-8 -*-

##
#    Controller for benchmarks
#         
#    I have determined that it will be more accurrate to use the same timing method for all benches
#    
#    @author     David Lasley <dave@dlasley.net>

from __future__ import division

import time
import csv
import logging

from subprocess import check_output

logging.basicConfig(level=logging.DEBUG)

def timeIt(func, loops=10):
    times = []
    for i in xrange(0, loops):
        start = time.time()
        func()
        times.append((time.time() - start) * 1000)
    return times

test_definitions = {
    'str concat': {
        'node': ['node', './node_benches.js', '--str'],
        'pypy': ['pypy', './py_benches.py', '--str'],
        'python': ['python', './py_benches.py', '--str'],
        'go': ['go', 'run', './go_benches.go', '-str'],
    },
    'recommended str concat': {
        'node': ['node', './node_benches.js', '--str'], #< Node's recommended is str+=str_
        'pypy': ['pypy', './py_benches.py', '--strrec'],
        'python': ['python', './py_benches.py', '--strrec'],
        'go': ['go', 'run', './go_benches.go', '-strrec'],
    },
    'int tests': {
        'node': ['node', './node_benches.js', '--int'],
        'pypy': ['pypy', './py_benches.py', '--int'],
        'python': ['python', './py_benches.py', '--int'],
        'go': ['go', 'run', './go_benches.go', '-int'],
    },
    'multi-d arr tests (fixed length golang)': {
        'node': ['node', './node_benches.js', '--arr'],
        'pypy': ['pypy', './py_benches.py', '--arr'],
        'python': ['python', './py_benches.py', '--arr'],
        'go': ['go', 'run', './go_benches.go', '-arr'],
    },
    'single-d arr tests (slice in golang)': {
        'node': ['node', './node_benches.js', '--slice'],
        'pypy': ['pypy', './py_benches.py', '--slice'],
        'python': ['python', './py_benches.py', '--slice'],
        'go': ['go', 'run', './go_benches.go', '-slice'],
    },
    'fibonacci': {
        'node': ['node', './node_benches.js', '--fib'],
        'pypy': ['pypy', './py_benches.py', '--fib'],
        'python': ['python', './py_benches.py', '--fib'],
        'go': ['go', 'run', './go_benches.go', '-fib'],
    },
}

with open('/tmp/bench_results.csv', 'w') as fh:
    
    w = csv.writer(fh)
    for test_name, test_set in test_definitions.iteritems():
        
        logging.info('Beginning %s tests' % test_name)
        w.writerow([test_name])
        
        for lng, cmd in test_set.iteritems():
                logging.info('Running %s' % lng)
                w.writerow([lng] + timeIt(lambda: check_output(cmd)))
            
        w.writerow([]) #< blank
