#!/usr/bin/env node
/*  Go benchmarks to compare against Python and Node.js
 *
 *  @author     Dave Lasley, dave@dlasley.net
 *  @website    https://blog.dlasley.net
 */

/*  Test str concatenation, generic method  */
function strConcat(){
    var str = '';
    
    for (var i=0; i<100000; i++){
        str += 'A';
    }
    
    return str
}

/*  Int math+reassignment   */
function intBench(){
    var test = 0;
    
    for(var i=0; i<100000; i++){
        test += i;
    }
    
    return test;
}

/*  Multidimensional Array assignments   */
function arrBench(){
    var test = [], test_ = [];
    for(var i=0; i<1000; i++){
        for(var j=0; j<100; j++){
            for(var k=0; k<100; k++) {
                test_[k] = i+j+k;
            }
            test[j] = test_;
        }
    }
    return test;
}

/*  Single dimensional array assignments    */
function sliceBench() {
    for(var i=0; i<100000; i++){
        var test = [];
        for(var j=0; j<100; j++){
            test.push(j+i);
        }
    }
    return test;
}

/*  Fib test    */
function fib(n) {
    if (n<2) {
        return n;
    }
    return fib(n-2) + fib(n-1);
}

var args = process.argv.slice(2);
switch (args[0]) {
    case '--str':
        console.log(strConcat());
        break;
    case '--int':
        console.log(intBench());
        break;
    case '--arr':
        console.log(arrBench());
        break;
    case '--slice':
        console.log(sliceBench());
        break;
    case '--fib':
        console.log(fib(40));
        break;
}