#!/usr/bin/pypy
# -*- coding: utf-8 -*-

##
#   Python benchmarks to compare against Go and Node.js
#         
#   @author     Dave Lasley, dave@dlasley.net
#   @website    https://blog.dlasley.net

from optparse import OptionParser

def strConcat():
    #   Test str concatenation, generic method
    str_ = ''
    
    for i in xrange(0, 100000):
        str_ += 'A'
        
    return str_

def strConcatRecommended():
    #   Language specific recommended way of str concat
    return ''.join(['A' for _ in xrange(0, 100000)])

def intBench():
    #   Int math+reassignment
    test = 0
    
    for i in xrange(0, 100000):
        test += i
        
    return test

def arrBench():
    #  Multidimensional Array assignments
    for i in xrange(0, 1000):
        test = []
        for j in xrange(0, 100):
            test.append([i+j+k for k in xrange(0, 100)])
            
    return test

def sliceBench():
    #   Single dimensional array assignments
    
    for i in xrange(0, 100000):
        test = []
        for j in xrange(0, 100):
            test.append(j+i)
            
    return test

def fib(n):
    #   Implement fibonacci
    if n<2:
        return n
    return fib(n-2) + fib(n-1)


if __name__ == '__main__':
    parser = OptionParser()
    parser.add_option('-s', '--str', dest='str', action="store_true", default=False, help='str concat test')
    parser.add_option('-r', '--strrec', dest='strrec', action="store_true", default=False, help='str join test')
    parser.add_option('-i', '--int', dest='int', action="store_true", default=False, help='int test')
    parser.add_option('-a', '--arr', dest='arr', action="store_true", default=False, help='array test')
    parser.add_option('-d', '--slice', dest='slice', action="store_true", default=False, help='slice test')
    parser.add_option('-f', '--fib', dest='fib', action="store_true", default=False, help='fib test')
    
    options, args = parser.parse_args()
    
    if options.str:
        print strConcat()
        
    if options.strrec:
        print strConcatRecommended()
        
    if options.int:
        print intBench()
        
    if options.arr:
        print arrBench()
        
    if options.slice:
        print sliceBench()
        
    if options.fib:
        print fib(40)